//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  DAA Strum Stick Fingering
//  Copyright (C) 2016 Jeffrey Edwards
//
//  created from / based on:
//  Note Names Plugin
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
//  Thanks to the makers of the Note Names Plugin
//  for making this possible
//
//  Interpretation:
//  black numbers - fret position on "A" string
//  black numbers under a "3" - fret position on "D" string
//  red "X" - note not available on "A" string
//  red "X" under a "3" - note not available on "D" string
//  gray "X" - note outside of playable range
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
   version: "1.0"
   description: "This plugin adds DAA Strum Stick Fingering"
   menuPath: "Plugins.DAA Strum Stick"

//=============================================================================
   function addFINGERING (cursor, note, posX, posY, isGrace) {
      //isGrace=1 > gracenote, isGrace=0 > note
      var text = newElement(Element.STAFF_TEXT);
      switch (isGrace){
         case 0: text.text = "<font size=\"14\"/>"; break;
         case 1: text.text = "<font size=\"10\"/>"; break;
      }// end for switch

      //determine the fingering for the note
      switch (note.pitch) {
         //notes on D(#3) string
         case 62: text.text = text.text +  "3\n" + "0";                     break; //D
         case 63: text.text = text.text +  "3\n" + "X"; text.color = "red"; break; //D# not available
         case 64: text.text = text.text +  "3\n" + "1";                     break; //E
         case 65: text.text = text.text +  "3\n" + "X"; text.color = "red"; break; //F  not available
         case 66: text.text = text.text +  "3\n" + "2";                     break; //F#
         case 67: text.text = text.text +  "3\n" + "3";                     break; //G
         case 68: text.text = text.text +  "3\n" + "X"; text.color = "red"; break; //G# not available
         //notes on A(#1) string
         case 69: text.text = text.text +  "0";                     break; //A
         case 70: text.text = text.text +  "X"; text.color = "red"; break; //A# not available
         case 71: text.text = text.text +  "1";                     break; //B
         case 72: text.text = text.text +  "X"; text.color = "red"; break; //C  not available
         case 73: text.text = text.text +  "2";                     break; //C#
         case 74: text.text = text.text +  "3";                     break; //D
         case 75: text.text = text.text +  "X"; text.color = "red"; break; //D# not available
         case 76: text.text = text.text +  "4";                     break; //E
         case 77: text.text = text.text +  "X"; text.color = "red"; break; //F  not available
         case 78: text.text = text.text +  "5";                     break; //F#
         case 79: text.text = text.text +  "6";                     break; //G
         case 80: text.text = text.text + "6½";                     break; //G#
         case 81: text.text = text.text +  "7";                     break; //A
         case 82: text.text = text.text +  "X"; text.color = "red"; break; //A# not available
         case 83: text.text = text.text +  "8";                     break; //B
         case 84: text.text = text.text +  "X"; text.color = "red"; break; //C  not available
         case 85: text.text = text.text +  "9";                     break; //C#
         case 86: text.text = text.text + "10";                     break; //D
         case 87: text.text = text.text +  "X"; text.color = "red"; break; //D# not available
         case 88: text.text = text.text + "11";                     break; //E
         case 89: text.text = text.text +  "X"; text.color = "red"; break; //F  not available
         case 90: text.text = text.text + "12";                     break; //F#
      } // end for switch

      //if note is outside playable range
      if ((note.pitch <  62) || (note.pitch > 90)) {
         text.text = text.text + "X";
         text.color = "gray";
      }

      text.pos.x=posX
      text.pos.y=posY
      cursor.add(text);
   }// end function addFINGERING

//=============================================================================

   onRun: {
      if (typeof curScore === 'undefined')
         Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
         fullScore = true;
         startStaff = 0; // start with 1st staff
         endStaff  = curScore.nstaves - 1; // and end with last
      } else {
         startStaff = cursor.staffIdx;
         cursor.rewind(2);
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1;
         } else {
            endTick = cursor.tick;
         }
         endStaff   = cursor.staffIdx;
      }

      for (var staff = startStaff; staff <= endStaff; staff++) {
         for (var voice = 0; voice < 4; voice++) {
               cursor.rewind(1); // beginning of selection
               cursor.voice    = voice;
               cursor.staffIdx = staff;

               if (fullScore)  // no selection
                  cursor.rewind(0); // beginning of score

               while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                  if (cursor.element && cursor.element.type == Element.CHORD) {

                     var notes = cursor.element.notes;
                     addFINGERING(cursor, notes[0], 0, 9, 0);

                     var graceChords = cursor.element.graceNotes;
                     // iterate through all grace chords
                     for (var i = 0; i < graceChords.length; i++) {
                        var notes = graceChords[i].notes;
                        // there seems to be no way of knowing the exact horizontal pos.
                        // of a grace note, so we have to guess:
                        // we need a better way of determining the Xposition of the gracenote
                        // until then adjust position manually
                        var posX = 2.5 * (graceChords.length - i);
                        addFINGERING(cursor, notes[0], posX, 9, 1);				
                     }

                  } // end if CHORD

                  cursor.next();
            } // end while segment
			
         } // end for voice
      } // end for staff
      Qt.quit();
   } // end onRun
} //end MuseScore
